/* Constants */

#define CMD_LENGTH 32

/* Variables */

char cmd[CMD_LENGTH];
char last[CMD_LENGTH];
bool repeat= false;

/* Functions */

void readCmd(char * cmd) {
  int val, pin;
  
  switch (cmd[0]) {
  case 'a': /* Read analog */
    Serial.print("analog ");
    pin= atoi(&cmd[1]);
    if (pin < 0 || pin >5) {
      Serial.print("on invalid pin ");
      Serial.println(pin);
    } else {
      Serial.println(pin);
      val= analogRead(pin);
      Serial.print("Value: ");
      Serial.println(val);
    } /* endif */
    break;
  case 'd': /* Read digital */
    Serial.print("digital ");
    pin= atoi(&cmd[1]);
    if (pin < 0 || pin >13) {
      Serial.print("on invalid pin ");
      Serial.println(pin);
    } else {
      Serial.println(pin);
      pinMode(pin, INPUT);
      val= digitalRead(pin);
      Serial.print("Value: ");
      Serial.println(val);
    } /* endif */
    break;
  default:
    Serial.print("Invalid read command: ");
    Serial.println(cmd[0]);
  } /* end switch */

} /* end readCmd */

/* Main Stuff */

void setup () {
  Serial.begin(9600);
  cmd[0]= '\0';
  last[0]= '\0';
}

void loop () {
  int length, delay;
  
  cmd[0]= '\0'; /* Reset command */
  
  length= Serial.readBytesUntil('\n', cmd, CMD_LENGTH);
  cmd[length]= '\0';
    
  if (length == 0 && repeat) {
     /* Repeat last command */
     strcpy(cmd, last);  
  } /* endif */
  
  if (cmd[0]) {
repeat:
    switch (cmd[0]) {
    case 'r': /* READ */
      Serial.print("Read ");
      readCmd(&cmd[1]);
      break;
    case 'w': /* WRITE */
      Serial.print("Write ");
      break;
    case 'a': /* Last command again */
      strcpy(cmd, last);
      goto repeat;
      break;
    case 'l': /* Repeat lsat command every # milliseconds */
      delay= atoi(&cmd[1]);
      if (delay == 0) { /* Deactivate loop */        
        Serial.println("loop deactivated");
        Serial.setTimeout(1000);
        repeat= false;
      } else {
        Serial.print("loop ");
        Serial.println(delay);
        Serial.println(last);
        Serial.setTimeout(delay);
        repeat= true;
      } /* endif */
      /* This command does not alter last command */
      strcpy(cmd, last);
      break;
    default:
      Serial.print("Unknown command: ");
      Serial.println(cmd);
    } /* end switch */
    
    strcpy(last, cmd);
  } /* endif */

} 

